package pps.core.prediction.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.prediction.entity.EnergyConsumptionAnalysisEnt;
import pps.core.prediction.entity.EnergyConsumptionAnalysisView;
import pps.core.prediction.mapper.EnergyConsumptionAnalysisMapper;
import pps.core.prediction.mapper.EnergyConsumptionAnalysisViewMapper;
import pps.core.prediction.service.data.energy_consumption_analysis.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

/**
 * 能耗分析模块(测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/11/23
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisOutput> getEnergyConsumptionAnalysis(XContext context, GetEnergyConsumptionAnalysisInput input) {
        EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
        QueryWrapper<EnergyConsumptionAnalysisEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(EnergyConsumptionAnalysisEnt::getId, input.getId());
        EnergyConsumptionAnalysisEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetEnergyConsumptionAnalysisOutput output = new GetEnergyConsumptionAnalysisOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisOutput> queryEnergyConsumptionAnalysis(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
        QueryWrapper<EnergyConsumptionAnalysisEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(EnergyConsumptionAnalysisEnt::getId, input.getId())
                .like(EnergyConsumptionAnalysisEnt::getLineId, input.getLineId())
                .eq(EnergyConsumptionAnalysisEnt::getPowerGeneration, input.getPowerGeneration())
                .eq(EnergyConsumptionAnalysisEnt::getPowerConsumption, input.getPowerConsumption())
                .eq(EnergyConsumptionAnalysisEnt::getDataDate, input.getDataDate());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryEnergyConsumptionAnalysisOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisOutput> dynamicQueryEnergyConsumptionAnalysis(XContext context, CustomQueryInput input) {
        EnergyConsumptionAnalysisMapper mapper = context.getBean(EnergyConsumptionAnalysisMapper.class);
        return XMapperHelper.query(mapper, input, EnergyConsumptionAnalysisEnt.class, QueryEnergyConsumptionAnalysisOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisViewOutput> getEnergyConsumptionAnalysisView(XContext context, GetEnergyConsumptionAnalysisViewInput input) {
        EnergyConsumptionAnalysisViewMapper mapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
        EnergyConsumptionAnalysisView record = new EnergyConsumptionAnalysisView();
        XCopyUtils.copyObject(input, record);
        EnergyConsumptionAnalysisView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetEnergyConsumptionAnalysisViewOutput output = new GetEnergyConsumptionAnalysisViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisViewInput input) {
        EnergyConsumptionAnalysisViewMapper mapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
        EnergyConsumptionAnalysisView record = new EnergyConsumptionAnalysisView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<EnergyConsumptionAnalysisView> list = mapper.selectList(record);
        PageInfo<EnergyConsumptionAnalysisView> pageInfo = new PageInfo<>(list);
        List<QueryEnergyConsumptionAnalysisViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryEnergyConsumptionAnalysisViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
